﻿using System;
using System.CodeDom;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms.VisualStyles;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Managers;
using ff14bot.Navigation;
using Kupo;
using Kupo.Settings;
using Pathfinding;
using TreeSharp;
using Action = TreeSharp.Action;
namespace Kupo.Rotations
{
    class Arcanist : KupoRoutine
    {
        public override void Initialize()
        {
            WindowSettings = new ArcanistSettings();
            base.Initialize();
        }



        private ArcanistSettings settings
        {
            get
            {
                return WindowSettings as ArcanistSettings;
            }
        }

        private string WhichPet
        {
            get
            {
                if (settings.PetKind == PetTypes.Caster)
                    return "Summon";

                if (settings.PetKind == PetTypes.Tank)
                    return "Summon II";

                if (settings.PetKind == PetTypes.Attacker)
                    return "Summon III";

                return "Summon";
            }
        }


        private string[] PullSpells = new[] { "Bio II", "Miasma", "Bio" };
        private string _BestPullSpell;
        private string BestPullSpell
        {
            get
            {
                if (string.IsNullOrEmpty(_BestPullSpell))
                {
                    foreach (var spell in PullSpells)
                    {
                        if (Actionmanager.HasSpell(spell))
                        {
                            _BestPullSpell = spell;
                            return spell;
                        }
                    }
                    _BestPullSpell = "Ruin";
                    return "Ruin";
                }
                else
                {
                    return _BestPullSpell;
                }
            }
        }


        public override int PullRange
        {
            get { return 25; }
        }

        public override ClassJobType[] Class
        {
            get { return new[] { ClassJobType.Arcanist, ClassJobType.Scholar, ClassJobType.Summoner }; }
        }


        protected override Composite CreatePreCombatBuffs()
        {
            return SummonChocobo();
        }

        protected override Composite CreateRest()
        {
            return new PrioritySelector(

                new Decorator(r => Core.Me.SpellCastInfo != null && Core.Me.SpellCastInfo.Name == "Physick" && !Resting, new Action(r => Actionmanager.StopCasting())),



                Apply("Aetherflow", r => Core.Player.CurrentManaPercent < 65 || (Resting && Core.Player.CurrentManaPercent < settings.RestEnergyDone), r => Core.Player),
                Cast("Physick", r => Resting && Core.Player.CurrentHealthPercent <= settings.RestHealthDone, r => Core.Player),
                DefaultRestBehavior(r => Core.Player.CurrentManaPercent)
                );
        }

        protected override Composite CreateHeal()
        {
            return new PrioritySelector(

                Apply("Aetherflow", r => Core.Player.CurrentManaPercent < 65, r => Core.Player),
                Cast("Physick", r => Core.Player.CurrentHealthPercent <= 40, r => Core.Player),
                Cast("Physick", r => Core.Player.Pet != null && Core.Player.Pet.CurrentHealthPercent <= settings.HealPet, r => Core.Player.Pet)
                );
        }

        protected override Composite CreatePull()
        {
            return new PrioritySelector(
                 EnsureTarget,

                 //Stop double casting due to delay
                 new Decorator(r => Core.Me.SpellCastInfo != null && Core.Me.SpellCastInfo.SpellData.Name.Equals(WhichPet) && Core.Player.Pet != null, new Action(r => Actionmanager.StopCasting())),
                Cast(r => WhichPet, r => Core.Player.Pet == null && Actionmanager.HasSpell(WhichPet), r => Core.Player),
                new Decorator(r => Actionmanager.InSpellInRangeLOS(BestPullSpell, Core.Target) == SpellRangeCheck.ErrorNotInRange, new Action(r => Navigator.MoveTo(Core.Target.Location))),
                new Decorator(r => Actionmanager.InSpellInRangeLOS(BestPullSpell, Core.Target) == SpellRangeCheck.ErrorNotInFront, new Action(r => Core.Target.Face())),
                Apply(r => BestPullSpell, r => Actionmanager.InSpellInRangeLOS(BestPullSpell, Core.Target) == SpellRangeCheck.Success)
                //new Decorator(r => Actionmanager.InSpellInRangeLOS("Ruin", Core.Target) == SpellRangeCheck.Success, new Action(r => Actionm))





            );
        }

        protected override Composite CreateCombat()
        {
            return new PrioritySelector(

                Cast(r => WhichPet, r => Core.Player.Pet == null && Actionmanager.HasSpell(WhichPet), r => Core.Player),
                Apply("Bio II"),
                Apply("Miasma"),
                Apply("Bio"),
                Cast("Energy Drain",r=> Core.Player.CurrentManaPercent < WindowSettings.RestEnergyDone),
                Cast("Ruin", r => true)


                );
        }



    }
}
